declare('afk', {})
afk.version  = '0.0.0'


afk.init          = {}
afk.log           = {}
afk.status        = false
afk.autoreply     = false
--afk.messages      = unspickle(LoadSystemNotes(-1)) or {}
afk.messages      = {}
afk.messages.list = {}
afk.ui            = {}
afk.use_fuzzy     = gkini.ReadInt('AFK', 'use_fuzzy', 1)
local namebox     = iup.hbox{}
local listbox     = iup.hbox{}

local tconcat     = table.concat
local tinsert     = table.insert
local tsort       = table.sort

dofile('fuzzytime.lua')

local function subdlg(ctrl)
	ctrl = ctrl or iup.fill{}
	return iup.dialog{
		ctrl,
		border = 'NO',
		menubox = 'NO',
		resize = 'NO',
--			bgcolor="0 0 0 0 *",
		active = 'NO'
	}
end

local font_color = '\127ffffff'

local bg = {
	[0] = "30 55 78 96",
	[1] = "42 74 96 96",
	[2] = "65 100 127 255",
}

-- ---GUI-------------------------------------------


local function update_lists()
	local msgs = afk.messages
	if (msgs.list) then tsort(msgs.list) end
	local names = msgs.list or {}
	local ui = afk.ui

	if (ui.names) then
		iup.Detach(ui.names)
		iup.Destroy(ui.names)
	end
	ui.names = iup.stationsublist{dropdown = 'YES', size = 150, action = function(self, t, i, v)
		test.print('\n\n-------------------')
		ui.messages:Get(t)
		iup.Refresh(ui.dialog)
	end}
	for i = 1, #names do
		ui.names[i] = names[i]
	end
	iup.Append(namebox, ui.names)

	ui.names:action(ui.names[1])
end

afk.ui.messages = iup.pdasubsublist{{}, control = 'YES', expand = 'YES', spacing = 0, value = 1, vo_listsel = 0, Get = function(self, name)
	test.print('Start - afk.ui.messages:Get')
	if (self.controls) then
		self[1] = nil
		for i = 1, #self.controls do
			iup.Detach(self.controls[i])
			iup.Destroy(self.controls[i])
		end
	end
	self.controls = {}
	local controls = self.controls
	local tbl = afk.messages[name]

	tsort(tbl, function(a,b) return a[1] < b[1] end)

	for i = 1, #tbl do
		test.print('['..i..'][1]='..tbl[i][1]..' / ['..i..'][2]'..tbl[i][2])
		local dlg = subdlg(iup.hbox{iup.label{title = tbl[i][1]}, iup.label{title = tbl[i][2], expand = 'YES'}, gap = 10})
		dlg['bgcolor'] = bg[i % 2]
		iup.Append(self, dlg)
		--dlg:map()
		iup.Refresh(dlg)
		controls[i] = dlg
	end
	self[1] = 1	
	iup.Refresh(self)
end}
afk.ui.messages[1] = 1
--afk.ui.messages['BGCOLOR'] = '65 100 127'

local function build_dialog()

	local main = iup.pdarootframe{
		iup.vbox{
			namebox,
			iup.hbox{afk.ui.messages,}, 
			gap = 3,
		},
		size="HALFxQUARTER",
		expand="NO",
	}

	afk.ui.dialog = iup.dialog{
		iup.vbox{
			iup.fill{},
			iup.hbox{
				iup.fill{},
				main,
				iup.fill{},
			},
			iup.fill{},
		},
		topmost = 'YES',
	}
	function afk.ui.dialog:Show()
		update_lists()
		ShowDialog(self)
	end
	afk.ui.dialog:map()
end

-- ---Core functionality--------------------------

function afk.log:OnEvent(_, data)
	if (not afk.status) then return end
	local name = data.name
	local msg = data.msg
	local messages = afk.messages

	if (not messages[name]) then 
		messages[name] = {}
		tinsert(messages.list, name)
	end

	tinsert(messages[name], {os.time(), msg})

	if (afk.autoreply) then SendChat(afk.autoreply, 'PRIVATE', name) end
end

function afk.output()
	local str = font_color
	local format = "%Y-%m-%d %H:%M:%S"

	local function format_time(time)
		if (type(time) ~= 'number') then return end
		if (afk.use_fuzzy == 1) then
			time = fuzzy.diff(os.time(), time)
		end

		if (type(time) == 'number') then return os.date(format, time)
		else return time
		end
	end

	for k, v in pairs(afk.messages) do
		if (k ~= 'list') then
			str = str..'Messages from '..k..'\n'
			for i = 1, #v do
				str = str..format_time(v[i][1])..' - '..v[i][2]..'\n' 
			end
			str = str..'\n'
		end
	end
	print(str)
end

function afk.init:OnEvent()
	UnregisterEvent(self, 'PLAYER_ENTERED_GAME')
	build_dialog()
end

RegisterEvent(afk.init, 'PLAYER_ENTERED_GAME')
RegisterEvent(afk.log, 'CHAT_MSG_PRIVATE')
RegisterUserCommand('afk', function(_, data)
	local help_str = font_color..
		'AFK usage:\n'..
		'\t/afk [message]\t- Toggle AFK status; optionally provide a message to auto-reply with\n'..
		'\t/afk view\t\t- Open the message viewer\n'..
		'\t/afk fuzzy\t\t- Toggles fuzzy time\n'..
		'\t/afk print\t\t- Print messages to the chat window'

	local function toggle_status(msg)
		local str = ''
		afk.status = not afk.status

		if (afk.status and msg) then 
			afk.autoreply = msg
			str = ' ("'..msg..'")'
		else 
			afk.autoreply = false 
		end

		local afk_map = {['true'] = 'now', ['false'] = 'no longer'}
		str = font_color..'You are '..afk_map[tostring(afk.status)]..' AFK'..str
		print(str)
	end

	if (not data) then 
		toggle_status()

	elseif (data[1] == 'help') then
		print(help_str)

	elseif (data[1] == 'view') then 
		afk.ui.dialog:Show()

	elseif (data[1] == 'print') then
		afk.output()

	elseif (data[1] == 'fuzzy') then
		local str = font_color..'Fuzzy time: '
		afk.use_fuzzy = -(afk.use_fuzzy - 1)
		gkini.WriteInt('AFK', 'use_fuzzy', afk.use_fuzzy)
		if (afk.use_fuzzy ==1) then str = str..'enabled' else str = str..'disabled' end
		print(str)
		
	else

		toggle_status(tconcat(data, ' '))
	end
end)
-- Event: CHAT_MSG_PRIVATEOUTGOING - Data: name="draugath",msg="test"
-- Event: CHAT_MSG_PRIVATE - Data: msg="test",name="draugath",faction=1
